@@ -14,10 +14,12 @@ MAX_PART_SIZE = MIB_5 * 5
 MAX_PARTS = 1000000
 MAX_PARALLEL = mp.cpu_count()
 MAX_RETRIES = 5
-HASHING_CHUNK_SIZE = 8192
+BATCH_PRESIGNS = 200
+MAX_PRESIGNS = 400
 
 CYCLE_SLEEP = 1    # progress bar refresh interval
 RETRY_SLEEP = 2    # sleep(RETRY_SLEEP * retry)
+SLOWDOWN_SLEEP = 4
 MON_TIMEOUT = 300
 WORKER_TIMEOUT = 60
 FORCED_GET_TIMEOUT = 0.1
