@@ -7,7 +7,7 @@
 
 """ OARepo S3 client lib. """
 
-import hashlib, re
+import hashlib, re, socket
 from os import path
 import time, requests, json, logging
 from urllib3.exceptions import NewConnectionError
@@ -36,16 +36,22 @@ class OARepoS3Client(object):
         self.parallel = MAX_PARALLEL if parallel == 0 else parallel
         self.quiet = quiet
         self.key = key
+        self.file = None
         self.contentType = 'application/octet-stream'
         self.parts, self.parts_unfin, self.uploadId, self.output = [], [], None, ''
         self.urlFiles = self.check_token_status(self.token)
+        self.checksum = None
+        self.presigns = None
+        self.nocheck = True
 
-    def process_click_upload(self, key=None, file=None):
+    def process_click_upload(self, key=None, file=None, nocheck=True):
+        self.nocheck = nocheck
         self.set_file(file, key)
         self.init_upload()
         return self.do_upload()
 
-    def process_click_resume(self, key, file, uploadId):
+    def process_click_resume(self, key, file, uploadId, nocheck=True):
+        self.nocheck = nocheck
         self.set_file(file, key)
         self.set_uploadId(uploadId)
         # parts = self.get_parts()
@@ -62,18 +68,30 @@ class OARepoS3Client(object):
             if self.results[partNum-1] is None:
                 self.parts_unfin.append(partNum)
         logger.debug(f"{funcname()} parts_unfin:\n{self.parts_unfin}")
+        self.presigns = SharedList(self.presign_parts_upload, self.parts_unfin, BATCH_PRESIGNS, MAX_PRESIGNS)
+        # ###
+        # secho(f"parts_unfin:{self.parts_unfin}", prefix='DBG', fg='red')
+        # secho(f"parts_unfin.join:{'#'.join(map(str,self.parts_unfin))}", prefix='DBG', fg='red')
         try:
             # parts_unfin = range(1, self.num_parts + 1)
             st = STATUS_OK
             if len(self.parts_unfin) > 0:
+                self.idle_callback(MAX_PRESIGNS)
                 self.parallels = Parallels(
-                    self.upload_part, self.num_parts, self.parts_unfin, parallel=self.parallel, quiet=self.quiet
+                    self.upload_part, self.idle_callback,
+                    self.num_parts, self.parts_unfin, parallel=self.parallel, quiet=self.quiet
                 )
                 st, newparts = self.parallels.main()
                 self.parts += newparts
+                # ###
+                # secho(f'\n>---- list:', quiet=self.quiet)
+                # for i in self.presigns.iter():
+                #     print(i, self.presigns.get_value(i))
+                # secho(f'\n<---- list end.', quiet=self.quiet)
             if st == STATUS_OK:
                 logger.debug(f"{funcname()} parts:\n{self.parts}")
                 location = self.complete_upload()
+                if not self.nocheck: self.process_click_check()
                 return location, STATUS_OK
             else:
                 raise Exception(f"Upload failed with status {st}.", st)
@@ -81,25 +99,31 @@ class OARepoS3Client(object):
             logger.debug(f"{funcname()} caught and raising Exception \"{e}\" {procname()}")
             raise e
 
-    def process_click_check(self, key, file):
-        self.set_file(file, key, showInfo=False)
-        msg = f"Checking file uploaded as key {self.key} with local file {file} ..."
+    def process_click_check(self, key=None, file=None):
+        if self.file is None or self.key is None: self.set_file(file, key, showInfo=False)
+        msg = f"Checking file uploaded as key {self.key} with local file {self.file} ..."
         secho(f"{msg}", quiet=self.quiet)
         urlFile = f"{self.urlFiles}{self.key}"
-        pool = mp.Pool(1)
-        fut_rem = pool.apply_async(get_remote_hash, args=(self.token, urlFile,))
-        pool.close()
+        if self.checksum is None:
+            pool = mp.Pool(1)
+            fut_rem = pool.apply_async(get_remote_hash, args=(self.token, urlFile, self.part_size,))
+            pool.close()
+        else:
+            secho(f"using ETag as remote checksum: {self.checksum}")
 
-        local_hash = get_local_hash(file)
-        logger.debug(f"\n local blake2b hash: {local_hash}")
+        local_hash = get_local_hash(self.file, self.part_size)
+        logger.debug(f"\n local checksum: {local_hash}")
+        # return True, STATUS_OK
 
-        remote_hash = fut_rem.get()
-        pool.join()
-        # remote_hash = get_remote_hash(self.token, urlFile)
+        if self.checksum is None:
+            pool.join()
+            remote_hash = fut_rem.get()
+        else:
+            remote_hash = self.checksum
 
-        logger.debug(f"\n remote blake2b hash: {remote_hash}")
+        logger.debug(f"\n remote checksum: {remote_hash}")
         if local_hash==remote_hash:
-            secho(f"Local and remote files have the same blake2b hash.",
+            secho(f"Local and remote files have the same checksum.",
                 prefix='OK', quiet=self.quiet)
             return True, STATUS_OK
         # return False, STATUS_GENERAL_ERROR
@@ -179,9 +203,11 @@ class OARepoS3Client(object):
         return uploadId
 
 
-    def presign_part_upload(self, partNum):
-        presign_url = f"{self.urlUpload}/{partNum}/presigned"
-        logger.debug(f"{funcname()} presign_part_upload (url:{presign_url})")
+    def presign_parts_upload(self, partNums):
+        pnstr = ",".join(map(str, partNums))
+        presign_url = f"{self.urlUpload}/{pnstr}/presigned"
+        # secho(f"{funcname()} {pnstr}")
+        logger.debug(f"{funcname()} presign_parts_upload (url:{presign_url})")
         try:
             resp = requests.get(presign_url, verify=self.https_verify)
             logger.debug(f"{funcname()} status: {resp.status_code}")
@@ -189,11 +215,13 @@ class OARepoS3Client(object):
                 raise Exception(f"Upload presign failed. (http code {resp.status_code})")
             logger.debug(f"{funcname()} status: {resp.json()}")
             resp_json = resp.json()
-            part_s3_url = resp_json['url'] if 'url' in resp_json.keys() else resp_json['presignedUrls'][str(partNum)]
-            logger.debug(f"{funcname()} part_s3_url: {part_s3_url}")
-            return part_s3_url
+            results = {}
+            for pn in resp_json['presignedUrls']:
+                results[int(pn)] = resp_json['presignedUrls'][pn]
+                logger.debug(f"{funcname()} part_s3_url[{pn}]: {resp_json['presignedUrls'][pn]}")
+            return results
         except Exception as e:
-            logger.debug(f"{funcname()} #{partNum} caught and raising Exception \"{e}\" {procname()}")
+            logger.debug(f"{funcname()} caught and raising Exception \"{e}\" {procname()}")
             raise type(e)(e.args).with_traceback(sys.exc_info()[2])
 
 
@@ -225,9 +253,14 @@ class OARepoS3Client(object):
         resp = requests.post(complete_url, data=parts4complete_json, headers=headers, verify=self.https_verify)
         logger.debug(f"{funcname()} status: {resp.status_code}")
         # logger.debug(f"{funcname()} resp.text: {resp.text}")
+        # secho(f"{funcname()} resp.text: {resp.text}")
         if resp.status_code >= 400:
             raise Exception(f"Upload completing failed (http code {resp.status_code})", STATUS_WRONG_SERVER_RESPONSE)
-        location = resp.json()['location']
+        rjson = resp.json()
+        location = rjson['location']
+        self.checksum = rjson['checksum'] if 'checksum' in rjson.keys() else None
+        logger.debug(f"Storage checksum={self.checksum}")
+        if self.checksum is not None: self.checksum = self.checksum.lstrip('etag:')
         logger.debug(f"{funcname()} location: {location}")
         secho(f'Upload completed. ({location})', prefix='OK', quiet=self.quiet)
         return location
@@ -272,7 +305,13 @@ class OARepoS3Client(object):
         logger.debug(f"\n>>Starting upload_part #{partNum} ...")
         offset = (partNum-1) * self.part_size
         part_size = self.part_size if partNum < self.num_parts else self.last_size
-        part_s3_url = self.presign_part_upload(partNum)
+        if not self.presigns.has_key(partNum):
+            secho(f"\n #{partNum} NOT in list: {partNum}")
+            # part_s3_url = self.presign_parts_upload([partNum])[partNum]
+            time.sleep(SLOWDOWN_SLEEP)
+        # else:
+        #    logger.debug(f"\n #{partNum} using {partNum} presign from cache")
+        part_s3_url = self.presigns.pop(partNum)
         ok = False
         # raise Exception('EXCEPTION')
         for retry in range(1, MAX_RETRIES + 1):
@@ -325,6 +364,8 @@ class OARepoS3Client(object):
         else:
             raise Exception(f"Part {partNum} upload failed.", STATUS_ERR_MAX_RETRIES)
 
+    def idle_callback(self, cnt=MAX_PRESIGNS):
+        self.presigns.supply(cnt, cnt)
 
     def logTest(self):
         logger.debug(f"debug")
