@@ -113,6 +113,52 @@ class Stats(object):
     def remaining(self):
         return self.num_parts - self.finished - self.failed
 
+class SharedList():
+    def __init__(self, action, unfin, grouplen, maxlen):
+        self.action = action
+        self.unfin = unfin
+        self.grouplen = grouplen
+        self.maxlen = maxlen
+        mpman = mp.Manager()
+        self.list = mpman.dict()
+        self.idx = 0
+
+    def prepare(self, cnt=0):
+        if cnt==0: cnt = self.maxlen
+        # cutting to smaller groups:
+        if cnt>self.grouplen:
+            for x in range(cnt // self.grouplen + (1 if (cnt % self.grouplen)>0 else 0)):
+                self.prepare(self.grouplen)
+            return
+        pnums = []
+        groupmax = min(self.idx+cnt, len(self.unfin))
+        if groupmax > self.idx:
+            for i in range(self.idx, groupmax):
+                pn = self.unfin[i]
+                pnums.append(pn)
+            vals = self.action(pnums)
+#            secho(f"\npresigns.prepare: {pnums} ({len(self.list)}/{self.maxlen})")
+            for pn in vals:
+                self.list[pn] = vals[pn]
+            self.idx = groupmax
+            # logger.debug(f" feeding list by {','.join(map(str,pnums))} idx={self.idx}")
+
+    def supply(self, cnt=0, min=0):
+        if min==0: min=self.grouplen
+        if len(self.list) < min:
+            self.prepare(cnt)
+
+    def iter(self):
+        return(self.list)
+
+    def has_key(self, pn):
+        return(pn in self.list)
+
+    def pop(self, pn):
+        return(self.list.pop(pn, None))
+
+    def get_value(self, pn):
+        return(self.list[pn])
 
 class Spinner(object):
     def __init__(self):
@@ -125,27 +171,29 @@ class Spinner(object):
         self.index = self.index + 1 if self.index + 1 < self.len else 0
         return spinchar
 
-def get_local_hash(file):
-    hashalg = hashlib.blake2b()
+def get_local_hash(file, _part_size=0):
+    hashes = []
+    part_size = _part_size if _part_size!=0 else MIN_PART_SIZE
     with open(file, "rb") as f:
         while 1:
-            chunk = f.read(HASHING_CHUNK_SIZE)
+            chunk = f.read(part_size)
             if not chunk: break
-            hashalg.update(chunk)
-    local_hash = hashalg.hexdigest()
+            hashes.append(hashlib.md5(chunk).digest())
+    local_hash = hashlib.md5(b''.join(hashes)).hexdigest() + '-' + str(len(hashes))
     return local_hash
 
-def get_remote_hash(token, url):
+def get_remote_hash(token, url, _part_size=0):
+    hashes = []
+    part_size = _part_size if _part_size!=0 else MIN_PART_SIZE
     headers = {
         'Authorization': f"Bearer {token}"
     }
     resp = requests.get(url, stream=True, headers=headers, verify=False)
     if resp.status_code >= 400:
         raise Exception(f"Can't read remote file.", STATUS_GENERAL_ERROR)
-    hashalg = hashlib.blake2b()
-    for data in resp.iter_content(HASHING_CHUNK_SIZE):
-        hashalg.update(data)
-    remote_hash = hashalg.hexdigest()
+    for chunk in resp.iter_content(part_size):
+        hashes.append(hashlib.md5(chunk).digest())
+    remote_hash = hashlib.md5(b''.join(hashes)).hexdigest() + '-' + str(len(hashes))
     return remote_hash
 
 
