@@ -12,13 +12,11 @@ from unittest.mock import Mock
 
 fake_file_size = 1024
 fake_data = ''.join(random.choices(string.hexdigits, k=fake_file_size))
-m = hashlib.blake2b()
-m.update(fake_data.encode())
 fake_file_info = Mock(
     size=fake_file_size,
     filename=''.join(random.choices(string.hexdigits, k=8)),
     data=fake_data,
-    hash_blake2b=m.hexdigest()
+    hash_md5=hashlib.md5(hashlib.md5(fake_data.encode()).digest()).hexdigest()+'-1'
 )
 
 @pytest.fixture(scope='module')
